/*
 * as5040.c
 *
 *  Created on: 22.04.2011
 *      Author: Wolfram
 *      These routine programs the zero position and reads sensor data from AS5040 rotary encoder
 *
 */
#include "as5040.h"
#include "uart.h"
#include <util/delay.h>
#include <avr/io.h>

uint16_t reverse_u16 (uint16_t in){	// reverts an 16 bit integer value
	int rev=0;
	  int i;

	  for (i = 0; i < 16; i++) {
	   rev = (rev << 1) | (in & 1);
	   in >>=1;
	  }
	  return rev;
}

void as5040_init()
{
	// set direction register (1=OUT 0=in)
	AS5040_DDR_CLK	= 1 ;		//    clock pin is output
	AS5040_DDR_DI  	= 0 ;		// -> data in  from AS5040 --> ATMEGA
	AS5040_DDR_PRG	= 1 ;		// -> data out from ATMEGA --> AS5040
	AS5040_DDR_CS	= 1 ;		//    chip select is output

	AS5040_CS = 0 ;
	AS5040_DI = 0 ;
	AS5040_PRG = 0 ;
	AS5040_CLK = 1 ;			// set to logic high, thus the chip starts already measuring
}

uint16_t as5040_ssi(void)
{
	uint16_t i, data;
	AS5040_CS 	= 1 ;
	_delay_us(2);
	AS5040_CS 	= 0 ;
	AS5040_CLK 	= 0;
	data 		= 0;

	for (i = 0; i < 16; i++)
	{
		AS5040_CLK = 1;
		_delay_us(1);
		AS5040_CLK = 0;							// conversion after falling edge CS

		data = data << 1;						// shift bits in target variable one to the left
		if ( AS5040_PORT & (1<<AS5040_PIN_DI) ) // read port bit
		{
			data |=0x01; 						// set new bit in 16 bit variable
		}
		_delay_us(1);
	}
	AS5040_CS = 1 ;

	return data;
}
void as5040_zero(uint16_t zpos )
{
	uint16_t 	i,otp;

	AS5040_PRG  = 0 ;
	AS5040_CLK 	= 0;
	AS5040_CS 	= 0 ;					// to program, chip select must be state high
	_delay_us(1);
	AS5040_PRG = 1 ;
	_delay_us(1);						// programming is done with rising edge of clk
	AS5040_CS 	= 1 ;
										// convert zero set point to otp format
	zpos = zpos & 0b0000001111111111;	// mask 10 bits and ignore higher bits
	zpos = zpos << 5;					// shift 5 bits left to leave room for the last 5 bits of programming scheme
	otp  = reverse_u16(zpos);			// and one final reverse, that s the complete OTP data stream

	for (i = 0; i < 16; i++)
	{
		AS5040_CLK 	= 0;				// clock = Low
		AS5040_PRG = otp & 0x01;		// prog. each status bit
		_delay_us(1);					// wait, could be decreased to 0.5us, but then the sensor does sometimes not answer correctly
		AS5040_CLK 	= 1;				// clock trigger -> high
		otp = otp >> 1;					// and next bit out
		_delay_us(1);					// wait
	}
	AS5040_CLK 	= 0;					// clock = Low
	AS5040_CS   = 0 ;					// reset CS to low again
}
