/*  Written by Wolfram Koerver 22.04.2011 if not otherwise specified
 *  Created on: 22.04.2011
 *  Test program to test the interface and subroutines for the
 *  AS5040 Rotary Encoder from Austria Microsystems
 *  The software uses UART routines from Peter Fleury <pfleury@gmx.ch>   http://jump.to/fleury
 */

#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <stdio.h>
#include <string.h>
#include <as5040.h>
#include "uart.h"

/* define CPU frequency in MHz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 8000000UL
#endif

/* ---------------- Portions necessary for UART support ----------------- */
#define	PRINT(string, ...)		printf_P(PSTR(string), ##__VA_ARGS__)
#define uchar unsigned char
#define uint unsigned int
#define UART_BAUD_RATE      38400
/* ----------------------------------------------------------------------- */

void outu16(unsigned int val)		// output unsigned integer values
{
	char 	outtext[12];
	utoa(val,outtext,10);
	outtext[11] = '\0';
	uart_puts(outtext);
}
void outu16b(unsigned int val)		// output unsigned integer values
{
	char 	outtext[17];
	utoa(val,outtext,2);			// 3rd Parameter = 10 = dezimal 2 = binary 16=hex
	outtext[16] = '\0';
	uart_puts(outtext);
}

void outu16b2(uint16_t val)			// filled with leading 0, output unsigned integer values
{
	int i;
	char buf[17];
	for (i = 0; i < 16; i++)
	  buf[15 - i] = (val & (1 << i)) ? '1' : '0';
	buf[16] = '\0';
	uart_puts(buf);
}

void outi16(unsigned int val)		// output signed integer values
{
	char 	outtext[12];
	itoa(val,outtext,10);			// 10 = Dezimal
	outtext[11] = '\0';
	uart_puts(outtext);
}

void delay_ms(uint16_t ms)
{
  for(uint16_t t=0; t<=ms; t++)
	_delay_ms(1);
}

void delay_1s(uint16_t d)
{
	uint8_t i;
	for (i = 0; i < d; i++)
	  delay_ms(1000);
}



int main (void) {
	uint16_t data,status,value;

    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) );	// Init the UART registers
	sei();													// Interrupts enable for UART routines
	uart_puts("AS5040 Rotary Routines Test\n\r");

	as5040_init();
	// ---------------------------------------------------------
	data	= as5040_ssi();							// read encoder values
	status	= data & 0b0000000000111111;
	value 	=(data & 0b1111111111000000) >> 6;		// mask and shift 6 bits to the right
	// ---------------------------------------------------------
	delay_ms(50);
	as5040_zero(0);								    // offset programming set 0 to last position
													// should only be programmed once after power up
	// ---------------------------------------------------------

	while(1)
    {
		delay_ms(50);
		// ---------------------------------------------------------
		data	= as5040_ssi();							// read encoder values
		status	= data & 0b0000000000111111;			// and extract status and angle
		value 	=(data & 0b1111111111000000) >> 6;		// mask and shift 6 bits to the right
		// ---------------------------------------------------------
		outu16(value);
		uart_puts("  ");
		outu16(((status & 0b0000000000000110)>>1 ));
		uart_puts("\r\n");
		// ---------------------------------------------------------
	}
}
